    red <- paste("<font color=\"red\">")             
    blue <- paste("<font color=\"blue\">")           
    plus1 <- paste("<font size=\"+0\">")           

shinyServer(function(input, output) {

    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b,a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) library(ggplot2)
#        if(gotLibs()[2]) library(grid)    
        if(!gotLibs()[3]) return(paste(red,"<h4>Warning: some R libraries not found, will use base graphics</h4>"))
        NULL       
    }) 
    
    digits <- reactive({
        if(input$which=="Type I") digits<-3
        if(input$which=="Type II") digits<-4
        if(input$which=="Type III") digits<-5
        if(input$which=="Type IV") digits<-2
        digits
    })    

    data <- reactive({ 
         if(input$which=="Type I") {
            x <- rnorm(input$n,10,3)
            y <- runif(1,-10,10)+rnorm(1,0,1)*x+rnorm(input$n,0,1)
         }       
         if(input$which=="Type II") {
            x <- rnorm(input$n,30,5)
            y <- runif(1,0,60)+rnorm(1,0,5)*x+rnorm(input$n,0,5)   
         }   
         if(input$which=="Type III") {
            x <- rnorm(input$n,0.5,0.1)
            y <- runif(1,0,2)+rnorm(1,0,0.5)*x+rnorm(input$n,0,0.5)            
         }
         if(input$which=="Type IV") {
            x <- rnorm(input$n,200,50)
            y <- runif(1,100,300)+rnorm(1,0,5)*x+rnorm(input$n,0,500)
         }
        
         data.frame(x=round(x,digits()-2),y=round(y,digits()-2))
    })              
 
  
   output$datatable <- renderText({
      x <- data()[,1]
      y <- data()[,2]
      ncol <- 20
      n <- length(x)
      m <- floor(n/ncol)
      
      lines <- as.list(1:ifelse(m*ncol<n,m+1,m))

      for(i in 1:m) {
          x1 <- as.character(x[((i-1)*ncol+1):(i*ncol)])
          y1 <- as.character(y[((i-1)*ncol+1):(i*ncol)])
          lines[[i]][1] <- "<table >"
          lines[[i]][2] <- paste("<tr align=\"right\"><th>&nbsp; x &nbsp;</th>",paste("<td>&nbsp;",x1,"&nbsp;</td>",collapse=""),"</tr>")
          lines[[i]][3] <- paste("<tr align=\"right\"><th>&nbsp; y&nbsp;</th>",paste("<td>&nbsp;",y1,"&nbsp;</td>",collapse=""),"</tr>")
          lines[[i]][4] <- "</table>"      
          lines[[i]][5] <- "<br>"      
      }    
      if(m*ncol<n) {
          x1 <- as.character(x[(m*ncol+1):n])
          y1 <- as.character(y[(m*ncol+1):n])              
          lines[[m+1]][1] <- "<table>"
          lines[[m+1]][2] <- paste("<tr align=\"right\"><th>&nbsp; x &nbsp;</th>",paste("<td>&nbsp;",x1,"&nbsp;</td>",collapse=""),"</tr>")
          lines[[m+1]][3] <- paste("<tr align=\"right\"><th>&nbsp; y &nbsp;</th>",paste("<td>&nbsp;",y1,"&nbsp;</td>",collapse=""),"</tr>")
          lines[[m+1]][4] <- "</table>"      
          lines[[m+1]][5] <- "<br>"      

      }

      unlist(lines)
   }) 
   
   output$text1 <- renderText({
        x <- data()[,1]
        y <- data()[,2]
        mu_x <- mean(x)
        tbl <- matrix(0,length(x),6)
        tbl[,1] <- x
        tbl[,2] <- round(x-mu_x,digits())
        tbl[,3] <- round((x-mu_x)/sd(x),digits())
        mu_y <- mean(y)
        tbl[,4] <- y
        tbl[,5] <- round(y-mu_y,digits())
        tbl[,6] <- round((y-mu_y)/sd(y),digits())
            
        line <- "<table border>"
        line[2] <- paste("<Tr><Th align=\"right\" ><strong>",plus1,"&nbsp;&nbsp;x</th>
                <th align=\"right\"><strong>",plus1,"&nbsp;&nbsp;",HTML("x-X&#773;"),"</th>
                <th align=\"right\"><strong>",plus1,"&nbsp;&nbsp;z<sub>x</sub></th>
                <Th align=\"right\"><strong>",plus1,"&nbsp;&nbsp;y</th>
                <th align=\"right\"><strong>",plus1,"&nbsp;&nbsp;",HTML("y-Y&#773;"),"</th>
                <th align=\"right\"><strong>",plus1,"&nbsp;&nbsp;z<sub>y</sub></th>
                </tr>")
        for(i in 1:length(x)) 
            line[i+2] <- paste("<Tr><Td align=\"right\">",plus1,"&nbsp;&nbsp;",tbl[i,1],"&nbsp;</td>
                <td align=\"right\">",plus1,"&nbsp;&nbsp;",tbl[i,2],"&nbsp;</td>
                <td align=\"right\">",plus1,"&nbsp;&nbsp;",tbl[i,3],"&nbsp;</td>
                <Td align=\"right\">",plus1,"&nbsp;&nbsp;",tbl[i,4],"&nbsp;</td>
                <td align=\"right\">",plus1,"&nbsp;&nbsp;",tbl[i,5],"&nbsp;</td>
                <td align=\"right\">",plus1,"&nbsp;&nbsp;",tbl[i,6],"&nbsp;</td>
                </tr>") 
            
        c(line,"</table>")    
   })        
   
   output$text2 <- renderText({
        x <- data()[,1]
        mu_x <- round(mean(x),digits())
        sd_x <- round(sd(x),digits())
        y <- data()[,2]
        mu_y <- round(mean(y),digits())
        sd_y <- round(sd(y),digits())
        line <- paste("<h4>",HTML("X&#773;"),"=",mu_x,
              HTML("&nbsp; Y&#773;"),"=",mu_y,"</h4>")
        line[2] <- paste("<h4>s<sub>x</sub>=",sd_x,
              "&nbsp;s<sub>y</sub>=",sd_y,"</h4>")
        line            
  })      
  
   output$text3 <- renderText({
        x <- data()[,1]
        y <- data()[,2]
        mu_x <- round(mean(x),digits())
        sd_x <- round(sd(x),digits())
        mu_y <- round(mean(y),digits())
        sd_y <- round(sd(y),digits())
        r <- round(sum((x-mu_x)/sd_x*(y-mu_y)/sd_y)/(length(x)-1),3)
        b1 <- round(r*sd_y/sd_x,digits())
        sgn <- ifelse(b1<0,"-","+")
        b0 <- round(mu_y-b1*mu_x,digits())
        line <- paste("<h4>r = (&Sum;z<sub>x</sub>z<sub>y</sub>)/(n-1)
              = ",r,"</h4>")
        line[2] <- paste("<h4>b<sub>1</sub> = 
                  r*s<sub>y</sub>/s<sub>x</sub> = ",
                  r,"*",sd_y,"/",sd_x," = ",b1,"</h4>")
        line[3] <- paste("<h4>b<sub>0</sub> = ", 
                  HTML("Y&#773;")," - b<sub>1</sub>",HTML("X&#773;"),
                  " = ",mu_y," - ",b1,"*",mu_x," = ",b0,"</h4>")
        line[4] <- paste("<h4>Least Squares Regression Line: y =",
                  round(b0,digits()-1),sgn,abs(round(b1,digits()-1)),"x</h4>")
        line
  })        
  
  output$plot <- renderPlot ({
      dta <- data()
      if(input$whichgraph=="base" | !gotLibs()[1]) {
          plot(x,y,xlab="x",ylab="y",pch=20)
          if(input$fit=="Yes")  abline(lm(y~x,lwd=3))
      }    
      else {
          plt <- ggplot(dta,aes(x,y))+geom_point()  
          if(input$fit=="Yes") plt <- plt+geom_smooth(method="lm",se=FALSE,size=1.2)
          print(plt)
      }    
  })         

   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#corcalc\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/22/2014</h6>")
      
  })

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo shows how to calculate a 
        correlation coefficient and the least squares regression line</h5>"
        
  })

})

